import asyncio
from concurrent.futures import ThreadPoolExecutor
import aiohttp
import json
import re
import urllib.parse
import logging
from logging.handlers import TimedRotatingFileHandler
import datetime
import threading
import websockets
import time  # 导入time模块

# 设置线程池大小
executor = ThreadPoolExecutor(max_workers=15)

# 最大重试次数
MAX_RETRIES = 3

# 获取当前日期
current_date = datetime.datetime.now().strftime("%Y-%m-%d")

# 创建日志文件的路径和文件名
log_filename = f"log_{current_date}.log"

# 设置日志记录器
logger = logging.getLogger("ClientLogger")
logger.setLevel(logging.INFO)

# 设置日志文件处理器，按日期分割日志文件
handler = TimedRotatingFileHandler(log_filename, when="midnight", interval=1, backupCount=7, encoding="utf-8")
handler.suffix = "%Y-%m-%d"  # 文件名后缀设置为日期

# 设置日志格式
formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
handler.setFormatter(formatter)

# 将处理器添加到日志记录器
logger.addHandler(handler)

# 创建线程锁
lock = threading.Lock()

async def send_response_to_server(response_data):
    server_url = "http://47.120.17.232:5000/receive_response"
    headers = {'Content-Type': 'application/json'}
    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(server_url, headers=headers, json=response_data) as response:
                # 此处可以处理响应
                pass
    except Exception as e:
        with lock:
            logger.error(f"Error sending response to server: {e}")

async def get_anti_content(cookie, href, ua, retries=0):
    api_url = f"https://wxapi.jjti.cn/bblServer?cookie={cookie}&href={href}&ua={ua}"
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(api_url) as response:
                if response.status == 200:
                    return await response.json()
                else:
                    with lock:
                        logger.error(f"Failed to get anti_content, status: {response.status}")
    except aiohttp.ClientError as e:
        with lock:
            logger.error(f"Request failed, attempt {retries + 1}: {e} ")

    if retries < MAX_RETRIES:
        await asyncio.sleep(1)
        return await get_anti_content(cookie, href, ua, retries + 1)
    else:
        with lock:
            logger.error("Failed to get anti_content after max retries.")
        return None

async def send_request(command, uid, token, show_id, live_share_token, page_from, command_id, live_url, retries=0):
    url = f"https://mobile.yangkeduo.com/proxy/api/api/fanta/chat/push"
    if uid:
        url += f"?pdduid={uid}"
    match = re.search(r"_(\d+)_", show_id)
    anchor_id = match.group(1) if match else None
    if not anchor_id:
        with lock:
            logger.error("Failed to get anchor id")
        if retries < MAX_RETRIES:
            await asyncio.sleep(1)
            return await send_request(command, uid, token, show_id, live_share_token, page_from, command_id, live_url, retries + 1)
        else:
            with lock:
                logger.error(f"Failed to get anchor id after max retries. command id {command_id}")
            return

    headers = {
        "Host": "mobile.yangkeduo.com",
        "Connection": "keep-alive",
        "Pragma": "no-cache",
        "Cache-Control": "no-cache",
        "Accept": "application/json, text/plain, */*",
        "User-Agent": "Mozilla/5.0 (Linux; Android 5.0; SM-G900P Build/LRX21T) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/33.0.0.0 Mobile Safari/537.36 MicroMessenger/6.0.0.54_r849063.501 NetType/WIFI",
        "Content-Type": "application/json;charset=UTF-8",
        "Origin": "https://mobile.yangkeduo.com",
        "Sec-Fetch-Site": "same-origin",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "document",
        "Accept-Encoding": "gzip, deflate, br, zstd",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
        "Cookie": f"PDDAccessToken={token}; "
    }

    anti_content = await get_anti_content(headers.get("Cookie"), live_url, headers.get("User-Agent"))
    if not anti_content:
        with lock:
            logger.error("Failed to get anti_content")
        if retries < MAX_RETRIES:
            await asyncio.sleep(1)
            return await send_request(command, uid, token, show_id, live_share_token, page_from, command_id, live_url, retries + 1)
        else:
            with lock:
                logger.error(f"Failed to get anti_content after max retries. command id {command_id}")
            return

    data = {
        "show_id": show_id,
        "anchor_id": anchor_id,
        "message": command,
        "page_from": page_from,
        "_live_share_token": live_share_token,
        "anti_content": anti_content,
        "command_id": command_id
    }

    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(url, headers=headers, json=data, ssl=False) as response:
                response_data = {
                    "status_code": response.status,
                    "response": await response.json() if response.status == 200 else await response.text(),
                    "command_id": command_id
                }
                await send_response_to_server(response_data)
    except aiohttp.ClientError as e:
        with lock:
            logger.error(f"Request failed: {e}")
        if retries < MAX_RETRIES:
            await asyncio.sleep(1)
            await send_request(command, uid, token, show_id, live_share_token, page_from, command_id, live_url, retries + 1)
        else:
            with lock:
                logger.error("Request failed after max retries.")

def handle_request_in_thread(data):
    asyncio.run(get_content_and_send(data))

async def handle_request(data):
    await asyncio.get_event_loop().run_in_executor(executor, handle_request_in_thread, data)

async def get_content_and_send(data):
    uid = data.get('uid')
    if uid is None or uid == '':
        with lock:
            logger.error(f"UID is empty, setting to None. data: {data}")
        uid = None
    encoded_url = data.get('live_url', '')
    decoded_url = urllib.parse.unquote(encoded_url)
    live_url = decoded_url.replace('\n', '')
    if not live_url:
        with lock:
            logger.error(f"Live url is empty, setting to None. data: {data}")

    # page_from_match = re.search(r'page_from=([^&]+)', live_url)
    # page_from = page_from_match.group(1) if page_from_match else None
    page_from = data.get('page_from')
    if not page_from:
        with lock:
            logger.error(f"Failed to get page from. data: {data}")

    # live_share_token_match = re.search(r'_live_share_token=([^&]+)', live_url)
    # live_share_token = live_share_token_match.group(1) if live_share_token_match else None
    live_share_token = data.get('live_share_token')
    if not live_share_token:
        with lock:
            logger.error(f"Failed to get live share token. data: {data}")

    token = data.get('token', '')
    if not token:
        with lock:
            logger.error("Failed to get token")

    command = data.get('command', '')
    if not command:
        with lock:
            logger.error(f"Failed to get command. data: {data}")

    show_id = data.get('show_id', '')
    if not show_id:
        with lock:
            logger.error(f"Failed to get show_id. data: {data}")

    command_id = data.get('command_id', '')
    if not command_id:
        with lock:
            logger.error(f"Failed to get command_id. data: {data}")

    await send_request(command, uid, token, show_id, live_share_token, page_from, command_id, live_url)

async def listen():
    uri = "ws://47.120.17.232:8765"
    while True:
        try:
            async with websockets.connect(uri) as websocket:
                print(f"Connected to {uri}")
                while True:
                    message = await websocket.recv()
                    try:
                        data = json.loads(message)
                        task = asyncio.create_task(handle_request(data))
                        tasks.append(task)
                    except json.JSONDecodeError as e:
                        with lock:
                            logger.error(f"Failed to decode JSON: {e}, message: {message}")
        except (websockets.exceptions.ConnectionClosedError, websockets.exceptions.ConnectionClosedOK) as e:
            with lock:
                logger.warning(f"Connection closed, reconnecting in 5 seconds: {e}")
        except Exception as e:
            with lock:
                logger.error(f"Unexpected error: {e}")
        await asyncio.sleep(5)  # Wait 5 seconds before attempting to reconnect

async def main():
    try:
        # 你的主程序逻辑
        await listen()
    except OSError as e:
        if "WinError" in str(e.winerror):
            logger.error(f"Encountered WinError: {e}")
            await asyncio.sleep(5)  # 等待5秒
            await main()  # 重新启动程序
        else:
            logger.error(f"Encountered unexpected error: {e}")
            raise

if __name__ == "__main__":
    tasks = []
    asyncio.run(main())
